package frames;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;

import drawing_objects.DrawingShip;
import generics.ShipsGenericCollection;
import movement_strategy.DrawingObjectShip;

public class FrameShipsCollection extends JFrame {
    private ShipsGenericCollection<DrawingShip, DrawingObjectShip> ships;
    JComponent pictureBoxCollection;
    TextField textFieldNumber;
    public FrameShipsCollection(){
        super("Набор кораблей");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        createGui();
        ships = new ShipsGenericCollection<>(pictureBoxCollection.getWidth(), pictureBoxCollection.getHeight());
        setVisible(true);
    }
    private void createGui(){
        //components initialisation
        pictureBoxCollection = new JComponent(){
            public void paintComponent(Graphics graphics){
                super.paintComponent(graphics);
                Graphics2D graphics2D = (Graphics2D) graphics;
                if (ships != null) ships.showShips(graphics2D);
                super.repaint();
            }
        };
        pictureBoxCollection.setPreferredSize(new Dimension(700, 450));
        JButton buttonAddShip = new JButton("Добавить корабль");
        textFieldNumber = new TextField();
        JButton buttonRemoveShip = new JButton("Удалить корабль");
        JButton buttonRefreshCollection = new JButton("Обновить коллекцию");
        //ActionListeners and ActionCommand addition
        buttonAddShip.addActionListener(e -> buttonAddShipClick());
        buttonRemoveShip.addActionListener(e -> buttonRemoveShipClick());
        buttonRefreshCollection.addActionListener(e -> buttonRefreshCollectionClick());
        //addition to frame
        JPanel panelCollection = new JPanel(new GridBagLayout());
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets.left = constraints.insets.right = 2;
        constraints.insets.top = constraints.insets.bottom = 25;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelCollection.add(buttonAddShip, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelCollection.add(textFieldNumber, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelCollection.add(buttonRemoveShip, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelCollection.add(buttonRefreshCollection, constraints);
        setLayout(new BorderLayout());
        add(panelCollection, BorderLayout.EAST);
        add(pictureBoxCollection, BorderLayout.CENTER);
        pack();
    }
    private void buttonAddShipClick() {
        FrameBattleship form;
        try {
            form = new FrameBattleship();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        form.selectShipButton.addActionListener(e->{
            form.select();
            DrawingShip ship = form.getSelectedShip();
            form.dispose();
            if (ships.insert(ship)) {
                JOptionPane.showMessageDialog(this, "Объект добавлен");
                pictureBoxCollection.repaint();
            }
            else {
                JOptionPane.showMessageDialog(this, "Не удалось добавить объект");
            }
        });
    }
    private void buttonRemoveShipClick(){
        int pos = Integer.parseInt(textFieldNumber.getText());
        if (ships.remove(pos)){
            JOptionPane.showMessageDialog(this, "Объект удален");
            pictureBoxCollection.repaint();
        }
        else{
            JOptionPane.showMessageDialog(this, "Не удалось удалить объект");
        }
    }
    private void buttonRefreshCollectionClick(){pictureBoxCollection.repaint();}
}
