package generics;

import drawing_objects.*;
import entities.EntityBattleship;
import entities.EntityShip;

import java.awt.*;
import java.util.Random;

public class HardGeneric<T extends EntityShip, U extends IDrawBlocks>{
    T[] ships;
    U[] blocks;
    private int shipsNumber;
    private int blocksNumber;
    private final int PICTURE_WIDTH;
    private final int PICTURE_HEIGHT;
    public HardGeneric(int shipsCount, int blocksCount, int width, int height){
        shipsNumber = 0;
        blocksNumber = 0;
        ships = (T[]) new EntityShip[shipsCount];
        blocks = (U[]) new IDrawBlocks[blocksCount];
        PICTURE_HEIGHT = height;
        PICTURE_WIDTH = width;
    }
    public boolean insertShip(T entityShip){
        if(ships[ships.length-1] != null)
            return false;
        for(int i = shipsNumber -1; i>= 0; i--) {
            ships[i + 1] = ships[i];
        }
        shipsNumber++;
        ships[0] = entityShip;
        return true;
    }
    public boolean insertBlock(U block){
        if(blocks[blocks.length-1] != null)
            return false;
        for(int i = blocksNumber -1; i>= 0; i--) {
            blocks[i + 1] = blocks[i];
        }
        blocksNumber++;
        blocks[0] = block;
        return true;
    }
    public DrawingShip makeObject(){
        Random rand = new Random();
        EntityShip entity = ships[rand.nextInt(0, shipsNumber)];
        IDrawBlocks block = blocks[rand.nextInt(0, blocksNumber)];
        if (entity instanceof EntityBattleship)
            return new DrawingBattleship(entity.getSpeed(), entity.getWeight(), entity.getBodyColor(), ((EntityBattleship) entity).getAdditionalColor(),
            ((EntityBattleship) entity).getTurret(), ((EntityBattleship) entity).getRocketLauncher(), PICTURE_WIDTH, PICTURE_HEIGHT, block.getType(), block.getNumber());
        return new DrawingShip(entity.getSpeed(), entity.getWeight(), entity.getBodyColor(), PICTURE_WIDTH, PICTURE_HEIGHT, block.getType(), block.getNumber());
    }
    public EntityShip makeRandomShip(){
        Random random = new Random();
        if (random.nextInt(2) == 0) {
            return new EntityShip(random.nextInt(100, 300), random.nextDouble(1000, 3000),
                    new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)));
        } else {
            return new EntityBattleship(random.nextInt(100, 300), random.nextDouble(1000, 3000),
                    new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                    new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                    random.nextBoolean(), random.nextBoolean());
        }
    }
    public IDrawBlocks makeRandomBlock(){
        Random random = new Random();
        IDrawBlocks block;
        switch (random.nextInt(3)){
            case 1 -> block = new DrawingRoundBlocks();
            case 2 -> block = new DrawingCrossBlocks();
            default -> block = new DrawingBlocks();
        }
        block.setNumber((random.nextInt(3) + 1) * 2);
        return block;
    }
}
