package generics;

import drawing_objects.*;
import movement_strategy.*;

import java.awt.*;

public class ShipsGenericCollection<T extends DrawingShip, U extends IMoveableObject>{
    private int pictureWidth;
    private int pictureHeight;
    private final int placeSizeWidth = 220;
    private final int placeSizeHeight = 60;
    private SetGeneric<T> collection;
    public ShipsGenericCollection(int picWidth, int picHeight){
        int width = picWidth / placeSizeWidth;
        int height = picHeight / placeSizeHeight;
        pictureWidth = picWidth;
        pictureHeight = picHeight;
        collection = new SetGeneric<T>(width * height, (Class<T>) DrawingShip.class);
    }
    public boolean insert ( T obj) {
        if (obj != null)
            return collection.insert(obj);
        return false;
    }
    public boolean remove (int pos){
        T obj = collection.get(pos);
        if (obj != null)
            return collection.remove(pos);
        return false;
    }
    public U getU(int pos){
        if(collection.get(pos) == null)
            return null;
        return (U)collection.get(pos).getMoveableObject();
    }
    public void showShips(Graphics2D graphics2D){
        DrawBackground(graphics2D);
        DrawObjects(graphics2D);
    }
    private void DrawBackground(Graphics2D g){
        BasicStroke stroke = new BasicStroke(3);
        g.setStroke(stroke);
        for (int i = 0; i < pictureWidth / placeSizeWidth; i++){
            for (int j = 0; j < pictureHeight / placeSizeHeight + 1; ++j){
                g.drawLine(i * placeSizeWidth, j * placeSizeHeight, i *
                        placeSizeWidth + placeSizeWidth / 2, j * placeSizeHeight);
            }
            g.drawLine(i * placeSizeWidth, 0, i * placeSizeWidth,
                    pictureHeight / placeSizeHeight * placeSizeHeight);
        }
    }
    private void DrawObjects(Graphics2D g){
        for (int i = 0; i < collection.getCount(); i++){
            DrawingShip ship = collection.get(i);
            if (ship != null)
            {
                int inRow = pictureWidth / placeSizeWidth;
                ship.setPosition(i % inRow * placeSizeWidth, (collection.getCount() / inRow - 1 - i / inRow) * placeSizeHeight + 5);
                ship.drawTransport(g);
            }
        }
    }
}
